package org.vcs.bazaar.eclipse.ui.preferences;

import java.io.File;

import org.eclipse.jface.preference.BooleanFieldEditor;
import org.eclipse.jface.preference.FieldEditorPreferencePage;
import org.eclipse.jface.preference.FileFieldEditor;
import org.eclipse.jface.preference.IntegerFieldEditor;
import org.eclipse.jface.preference.PathEditor;
import org.eclipse.jface.preference.StringFieldEditor;
import org.eclipse.jface.util.PropertyChangeEvent;
import org.eclipse.ui.IWorkbench;
import org.eclipse.ui.IWorkbenchPreferencePage;
import org.vcs.bazaar.client.BazaarClientFactory;
import org.vcs.bazaar.client.BazaarPreference;
import org.vcs.bazaar.client.core.BazaarClientException;
import org.vcs.bazaar.eclipse.ClientProperties;
import org.vcs.bazaar.eclipse.EclipseBazaarCore;
import org.vcs.bazaar.eclipse.internal.core.BazaarException;
import org.vcs.bazaar.eclipse.ui.EclipseBazaarUI;
import org.vcs.bazaar.eclipse.ui.UITexts;

/**
 * Main Preference page, it allow configure: <br>
 * - the bzr executable path<br>
 * - plugins path <br>
 * - xmlrpc service port<br>
 * 
 * @author Guillermo Gonzalez <guillo.gonzo@gmail.com>
 */
public class BazaarPreferencePage extends FieldEditorPreferencePage implements IWorkbenchPreferencePage {

    private FileFieldEditor executable;
    private BooleanFieldEditor overwriteEmail;
    private StringFieldEditor email;
    private PathEditor pluginPath;
    private StringFieldEditor clientValue;
	private IntegerFieldEditor xmlrpcPort;

	public BazaarPreferencePage() {
        super(GRID);
		setPreferenceStore(EclipseBazaarUI.getDefault().getPreferenceStore());
		setDescription(UITexts.BazaarPreferencesPage_description);
	}

	public void init(IWorkbench workbench) {}

	/**
	 * Creates the field editors. Field editors are abstractions of the common
	 * GUI blocks needed to manipulate various types of preferences. Each field
	 * editor knows how to save and restore itself.
	 */
	@Override
    public void createFieldEditors() {
		executable = new FileFieldEditor(BazaarPreference.EXECUTABLE.name(), UITexts.BazaarPreferencesPage_executable, getFieldEditorParent());
		overwriteEmail = new BooleanFieldEditor(BazaarPreferenceConstants.OVERWRITE_EMAIL,
                UITexts.BazaarPreferencesPage_overwrite_userid, getFieldEditorParent());
		email = new StringFieldEditor(BazaarPreference.BZR_EMAIL.name(), UITexts.BazaarPreferencesPage_userid, getFieldEditorParent());
        pluginPath = new PathEditor(BazaarPreference.BZR_PLUGIN_PATH.name(), UITexts.BazaarPreferencesPage_plugins_dir, UITexts.BazaarPreferencesPage_plugins_dir_chooser, getFieldEditorParent());
		addField(executable);
		addField(overwriteEmail);
		addField(email);
		addField(pluginPath);
		createClientTypeControl();
		createXmlRpClientConfigControl();
		executable.getTextControl(getFieldEditorParent()).setEditable(false);
		email.getTextControl(getFieldEditorParent()).setEnabled(getPreferenceStore().getBoolean(BazaarPreferenceConstants.OVERWRITE_EMAIL));
	}

	@Override
	public void propertyChange(PropertyChangeEvent event) {
		super.propertyChange(event);
		if (event.getSource() == overwriteEmail) {
			email.getTextControl(getFieldEditorParent()).setEnabled((Boolean)event.getNewValue());
		}
	}
	
	private void createClientTypeControl() {
		clientValue = new StringFieldEditor("ClientValue", UITexts.BazaarPreferencesPage_client_type, getFieldEditorParent()); //$NON-NLS-1$
		addField(clientValue);
		clientValue.getTextControl(getFieldEditorParent()).setEnabled(false);
		setClientTypeLabel();
	}

	private void createXmlRpClientConfigControl() {
		xmlrpcPort = new IntegerFieldEditor(BazaarPreference.BZR_XMLRPC_PORT.name(), UITexts.BazaarPreferencesPage_xmlrpc_port, getFieldEditorParent(), 5);
		addField(xmlrpcPort);
	}

	private void setClientTypeLabel() {
		try {
			clientValue.setStringValue(BazaarClientFactory.getPreferredClientType());
		} catch (BazaarClientException e1) {
			clientValue.setStringValue(e1.getMessage());
		}
	}

	@Override
	protected void initialize() {
		super.initialize();
		performOk();
	}

	@Override
	protected void performDefaults() {
		super.performDefaults();
		email.getTextControl(getFieldEditorParent()).setEnabled(overwriteEmail.getBooleanValue());
	}

	@Override
	public boolean performOk() {
		if (super.performOk()) {
			return checkValidClient();
		}
		return false;
	}

	private boolean checkValidClient() {
        final StringBuilder pluginPath = new StringBuilder();
        for (String item : this.pluginPath.getListControl(getFieldEditorParent()).getItems()) {
            if (pluginPath.length() > 0) {
                pluginPath.append(File.pathSeparatorChar);
            }
            pluginPath.append(item);
        }
        EclipseBazaarCore.getDefault();
        ClientProperties properties = new ClientProperties(executable.getStringValue(), overwriteEmail.getBooleanValue() ? email.getStringValue() : "", Integer.parseInt(xmlrpcPort.getStringValue()), pluginPath.toString());
        EclipseBazaarCore.getDefault().setClientProperties(properties);
		// check the client state
		try {
			boolean result = EclipseBazaarCore.getDefault().checkClient();
			setMessage(null, ERROR);
			return result;
		} catch (BazaarException e) {
			setMessage(e.getStatus().getMessage(), ERROR);
			return false;
		} finally {
			setClientTypeLabel();
		}
	}

}